"""
Tests for Generator
"""
import io
import os
import shutil
import sys
import tempfile
from unittest import TestCase, mock

import pytest
from django.conf import settings

from dr_scaffold.generators import Generator, pluralize
from dr_scaffold.scaffold_templates import model_templates, serializer_templates


class TestGenerator(TestCase):
    """
    Tests for Generator
    """

    test_settings = settings
    tmpdirpath = tempfile.mkdtemp()
    # tmpdirpath = "generated_tests_folder"
    core_folder = f"{tmpdirpath}/{test_settings.CORE_FOLDER}"
    api_folder = f"{tmpdirpath}/{test_settings.API_FOLDER}"

    def setUp(self):
        """
        Tests set up : get executed before each test
        """
        os.makedirs(self.tmpdirpath, exist_ok=True)
        os.makedirs(f"{self.core_folder}blog", exist_ok=True)
        os.makedirs(f"{self.api_folder}blog", exist_ok=True)
        for file_name in [
            f"{self.core_folder}blog/models.py",
            f"{self.api_folder}blog/serializers.py",
            f"{self.core_folder}blog/admin.py",
            f"{self.api_folder}blog/urls.py",
            f"{self.api_folder}blog/views.py",
        ]:
            with open(file_name, "x", encoding="utf8") as file:
                file.close()

    def tearDown(self):
        """
        Tests tear down : get executed after each test
        """
        for file_name in [
            f"{self.core_folder}blog/models.py",
            f"{self.api_folder}blog/serializers.py",
            f"{self.core_folder}blog/admin.py",
            f"{self.api_folder}blog/urls.py",
            f"{self.api_folder}blog/views.py",
        ]:
            os.remove(file_name)
        if os.path.exists(self.tmpdirpath):
            shutil.rmtree(self.tmpdirpath)

    @classmethod
    def test_pluralize(cls):
        """
        Test pluralization
        """
        assert pluralize("article") == "articles"
        assert pluralize("category") == "categories"
        assert pluralize("post") == "posts"

    @classmethod
    def test_init(cls):
        """
        Test arguments
        """
        generator_obj = Generator("blog", "Article", "")
        assert generator_obj.app_name == "blog"
        assert generator_obj.model_name == "Article"

    def test_add_setup_imports(self):
        """
        Tests imports
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_folder = self.core_folder
        file_paths = (f"{self.core_folder}blog/models.py",)
        matching_imports = (serializer_templates.SETUP,)
        generator_obj.add_setup_imports(file_paths, matching_imports)
        with open(file_paths[0], "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert body == serializer_templates.SETUP

    def test_setup_files(self):
        """
        Tests files setup
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.setup_files()
        core_files = list(os.listdir(f"{self.core_folder}blog/"))
        api_files = list(os.listdir(f"{self.api_folder}blog/"))
        with open(f"{self.core_folder}blog/models.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert len(core_files + api_files) == 7  # taking in count migrations folder
        assert ("from django.db import models" in body) is True

    @classmethod
    @mock.patch("dr_scaffold.generators.file_api.is_present_in_file")
    def test_get_fields_string(cls, mock_is_in_file):
        """
        Tests
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        fields_string = generator_obj.get_fields_string(generator_obj.fields)
        body_template = model_templates.TEXTFIELD % dict(name="body")
        title_template = model_templates.CHARFIELD % dict(name="title")
        string = title_template + body_template
        assert fields_string == string
        # test relation field type
        generator_obj2 = Generator("blog", "Article", ("author:foreignkey:Author",))
        generator_obj2.get_fields_string(generator_obj2.fields)
        mock_is_in_file.assert_called_once()

    def test_get_fields_string_relation_model_not_created_yet(self):
        """
        Tests
        """
        generator_obj2 = Generator("blog", "Article", ("author:foreignkey:Author",))
        generator_obj2.core_dir = self.core_folder
        generator_obj2.api_dir = self.api_folder
        captured_output = io.StringIO()
        sys.stdout = captured_output
        generator_obj2.get_fields_string(generator_obj2.fields)
        sys.stdout = sys.__stdout__
        assert "⚠️ bare in mind that Author" in captured_output.getvalue()

    @classmethod
    def test_get_model_string(cls):
        """
        Tests
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        fields_string = generator_obj.get_fields_string(fields=generator_obj.fields)
        model_string = generator_obj.get_model_string()
        string = model_templates.MODEL % ("Article", fields_string, "Articles")
        assert model_string == string

    def test_run(self):
        """
        Tests
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        result = generator_obj.run()
        assert result == print("🎉 Your RESTful Article api resource is ready 🎉")
        # test case where something is wrong
        generator_obj2 = Generator("blog", "Author", ("title:charfiel",))
        generator_obj2.core_dir = self.core_folder
        generator_obj2.api_dir = self.api_folder
        result = generator_obj2.run()
        assert result == print("🤔 Oops something is wrong: charfiel")

    @mock.patch("dr_scaffold.generators.Generator.generate_app")
    def test_generate_api(self, mock_generate_app):
        """
        Tests
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_api()
        mock_generate_app.assert_called_once()

    def test_generate_models(self):
        """
        Tests
        """
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_models()
        with open(f"{self.core_folder}blog/models.py", "r+", encoding="utf8") as file:
            body = "".join(line for line in file.readlines())
        assert ("Article" in body) is True
        assert ("Articles" in body) is True
        assert ("title" in body) is True
        assert ("body" in body) is True
        # test when model already generated
        generator_obj = Generator(
            "blog", "Article", ("title:charfield", "body:textfield")
        )
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_models()
        assert body.count("class Article") == 1
        assert body.count("class Meta:") == 1

    def test_get_admin_parts(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        head, body = generator_obj.get_admin_parts()
        assert ("import Article" in head) is True
        assert ("@admin.register(Article)" in body) is True

    def test_register_models_to_admin(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.register_models_to_admin()
        with open(f"{self.core_folder}blog/admin.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert ("import Article" in body) is True
        assert ("@admin.register(Article)" in body) is True
        # test when model already registered
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.register_models_to_admin()
        with open(f"{self.core_folder}blog/admin.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert body.count("models import Article") == 1
        assert body.count("@admin.register(Article)") == 1

    def test_get_viewset_parts(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        head, body = generator_obj.get_viewset_parts()
        assert ("import Article" in head) is True
        assert ("class ArticleViewSet" in body) is True

    def test_generate_views(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_views()
        with open(f"{self.api_folder}blog/views.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert ("import Article" in body) is True
        assert ("ArticleViewSet" in body) is True
        assert ("ArticleSerializer" in body) is True
        # test if view already exists
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_views()
        with open(f"{self.api_folder}blog/views.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert body.count("models import Article") == 1
        assert body.count("ArticleViewSet") == 1
        assert body.count("import ArticleSerializer") == 1
        assert body.count("serializer_class = ArticleSerializer") == 1

    def test_get_serializer_parts(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        head, body = generator_obj.get_serializer_parts()
        assert ("import Article" in head) is True
        assert ("class ArticleSerializer" in body) is True

    def test_generate_serializer(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_serializers()
        with open(
            f"{self.api_folder}blog/serializers.py", "r+", encoding="utf8"
        ) as file:
            body = "".join(file.readlines())
        assert ("import Article" in body) is True
        assert ("ArticleSerializer" in body) is True
        # test if serializer already exists
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_serializers()
        with open(
            f"{self.api_folder}blog/serializers.py", "r+", encoding="utf8"
        ) as file:
            body = "".join(file.readlines())
        assert body.count("import Article") == 1
        assert body.count("ArticleSerializer") == 1

    def test_get_url_parts(self):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        head, body = generator_obj.get_url_parts()
        assert ("import ArticleViewSet" in head) is True
        assert (", ArticleViewSet)" in body) is True

    @mock.patch("dr_scaffold.file_api.replace_file_chunk")
    def test_generate_urls(self, mock_replace_chunk):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_urls()
        with open(f"{self.api_folder}blog/urls.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert ("import ArticleViewSet" in body) is True
        assert (", ArticleViewSet)" in body) is True
        # test : if url have been added we won't add it again
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_urls()
        with open(f"{self.api_folder}blog/urls.py", "r+", encoding="utf8") as file:
            body = "".join(file.readlines())
        assert body.count("import ArticleViewSet") == 1
        assert body.count(", ArticleViewSet)") == 1
        # test : if when creating another resource we replace the url_patterns
        generator_obj = Generator("blog", "Author", ("name:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_urls()
        mock_replace_chunk.assert_called()

    @mock.patch("dr_scaffold.generators.Generator.generate_api")
    def test_generate_api_called(self, mock_generate_api):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_api()
        mock_generate_api.assert_called()

    @mock.patch("dr_scaffold.generators.Generator.run")
    def test_run_called(self, mock_run):
        """
        Tests
        """
        generator_obj = Generator("blog", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.run()
        mock_run.assert_called()

    @mock.patch("dr_scaffold.generators.Generator.setup_files")
    def test_generate_app(self, mock_setup_files):
        """
        Tests
        """
        generator_obj = Generator("blog2", "Article", ("title:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_app()
        mock_setup_files.assert_called()

    @mock.patch("dr_scaffold.generators.Generator.setup_files")
    def test_generate_app_appdir_exist(self, mock_setup_files):
        """
        Tests
        """
        generator_obj = Generator("blog", "Author", ("name:charfield",))
        generator_obj.core_dir = self.core_folder
        generator_obj.api_dir = self.api_folder
        generator_obj.generate_app()
        mock_setup_files.assert_not_called()

    def test_get_folder_settings_not_set(self):
        """
        if no settings folder paths exists
        """
        delattr(self.test_settings, "CORE_FOLDER")
        delattr(self.test_settings, "API_FOLDER")
        generator = Generator("blog", "Author", ("name:charfield",))
        assert generator.core_dir == ""
        assert generator.api_dir == ""

    def test_get_folder_settings_without_forward_slash(self):
        """
        test add forward slash if forgotten
        """
        # overwriting settings for test
        setattr(self.test_settings, "CORE_FOLDER", "core")
        setattr(self.test_settings, "API_FOLDER", "api")
        with pytest.raises(ValueError, match=r".* should end with .*"):
            Generator("blog", "Author", ("name:charfield",))
        # settings to normal
        setattr(self.test_settings, "CORE_FOLDER", "my_app_core/")
        setattr(self.test_settings, "API_FOLDER", "my_app_api/")

    def test_get_folder_settings_with_forward_slash(self):
        """
        if settings are well set
        """
        setattr(self.test_settings, "CORE_FOLDER", "core/")
        setattr(self.test_settings, "API_FOLDER", "api/")
        generator_obj = Generator("blog", "Author", ("name:charfield",))
        assert generator_obj.core_dir == "core/"
        assert generator_obj.api_dir == "api/"



"""
templates for views
"""

VIEWSET = """class %(model)sViewSet(viewsets.ModelViewSet):
    queryset = %(model)s.objects.all()
    serializer_class = %(model)sSerializer
    

"""

MODEL_IMPORT = """from %(app)s.models import %(model)s
"""

SERIALIZER_IMPORT = """from %(app)s.serializers import %(model)sSerializer
"""

SETUP = """from rest_framework import viewsets

"""



"""
templates for admin
"""

REGISTER = """@admin.register(%(model)s)
class %(model)sAdmin(admin.ModelAdmin):
    exclude = ()
    
"""
MODEL_IMPORT = """from %(app)s.models import %(model)s
"""

SETUP = """from django.contrib import admin

"""



"""
templates for serializers
"""

SERIALIZER = """class %(model)sSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = %(model)s
        fields = '__all__'
        

"""

MODEL_IMPORT = """from %(app)s.models import %(model)s
"""

SETUP = """from rest_framework import serializers

"""



"""
A file api created only for file operations extracted from Generator
"""
from os import path


def create_file(file_path):
    """
    creates a file
    """
    with open(file_path, "x", encoding="utf8"):
        pass


def wipe_file_content(file_path):
    """
    wipes a file content
    """
    with open(file_path, "r+", encoding="utf8") as file:
        file.truncate(0)


def get_file_content(file_path):
    """
    gets a file content
    """
    with open(file_path, "r+", encoding="utf8") as file:
        return "".join(file.readlines())


def set_file_content(file_path, body):
    """
    sets a file content
    """
    wipe_file_content(file_path)
    with open(file_path, "r+", encoding="utf8") as file:
        file.write(body)


def prepend_file_content(file_path, head):
    """
    prepends a string to file
    """
    body = get_file_content(file_path)
    wipe_file_content(file_path)
    set_file_content(file_path, head + body)


def append_file_content(file_path, tail):
    """
    appends a string to file
    """
    body = get_file_content(file_path)
    wipe_file_content(file_path)
    set_file_content(file_path, body + tail)


def wrap_file_content(file_path, head, tail):
    """
    wraps a file content between two strings and sets the result as the content of the file
    """
    prepend_file_content(file_path, head)
    append_file_content(file_path, tail)


def replace_file_chunk(file_path, chunk, new):
    """
    replaces a string in a file
    """
    body = get_file_content(file_path)
    body = body.replace(chunk, new)
    wipe_file_content(file_path)
    set_file_content(file_path, body)


def is_present_in_file(file_path, string):
    """
    check for string if present in a file
    """
    body = get_file_content(file_path)
    return string in body


def wipe_files(file_paths):
    """
    wipe files
    """
    for file in file_paths:
        wipe_file_content(file)


def create_files(file_paths):
    """
    creates files
    """
    for file in file_paths:
        if not path.isfile(file):
            create_file(file)



"""
Generator module where the real work happens
"""
from os import makedirs, path

import inflect
from django.conf import settings

from dr_scaffold import file_api
from dr_scaffold.scaffold_templates import (
    admin_templates,
    app_template,
    model_templates,
    serializer_templates,
    url_templates,
    view_templates,
)


def pluralize(string):
    """
    pluralizes a string word using a python library, needed for verbose model names and url paths
    """
    pluralizer = inflect.engine()
    return pluralizer.plural(string)


class Generator:
    """
    A wrapper for CLI command arguments and the REST api different files generation methods
    """

    def __init__(self, app_name, model_name, fields):
        """
        :param appdir: a string that has the app directory's path or just the appname itself
        :param model_name: model name we want to generate
        :param fields: a list of fields strings we want to generate along
        """
        self.init_paths_from_settings()
        self.app_name = app_name
        self.model_name = model_name
        self.fields = fields

    def run(self):
        """
        runs generate api and throws an exception if something went wrong
        """
        try:
            self.generate_api()
        except Exception as error:
            return print(f"🤔 Oops something is wrong: {error}")
        return print(f"🎉 Your RESTful {self.model_name} api resource is ready 🎉")

    def generate_api(self):
        """
        Generates a REST api files based on CLI command arguments
        """
        self.generate_app()
        self.generate_models()
        self.register_models_to_admin()
        self.generate_serializers()
        self.generate_views()
        self.generate_urls()

    @classmethod
    def add_setup_imports(cls, file_paths, matching_imports):
        """
        adds the bare minimum imports needed for each file
        """
        for i, file_path in enumerate(file_paths):
            file_api.set_file_content(file_path, matching_imports[i])

    def setup_folders(self):
        """
        creates folders if they not exist
        """
        core_app_path = self.core_dir + self.app_name
        api_app_path = self.api_dir + self.app_name
        migrations_path = f"{core_app_path}/migrations/"
        makedirs(core_app_path, exist_ok=True)
        makedirs(api_app_path, exist_ok=True)
        makedirs(migrations_path, exist_ok=True)

    def setup_files(self):
        """
        creates files if not exist, and adds appropriate imports
        """
        self.setup_folders()
        core_app_path = self.core_dir + self.app_name
        api_app_path = self.api_dir + self.app_name
        apps_template = app_template.TEMPLATE % (
            self.app_name.capitalize(),
            core_app_path.replace("/", "."),
        )
        files = (
            f"{api_app_path}/serializers.py",
            f"{api_app_path}/urls.py",
            f"{core_app_path}/models.py",
            f"{core_app_path}/admin.py",
            f"{api_app_path}/views.py",
            f"{core_app_path}/apps.py",
        )
        files_matching_imports = (
            serializer_templates.SETUP,
            url_templates.SETUP,
            model_templates.SETUP,
            admin_templates.SETUP,
            view_templates.SETUP,
            apps_template,
        )
        file_api.create_files(files)
        if not path.exists(f"{core_app_path}/migrations/__init__.py"):
            file_api.create_file(f"{core_app_path}/migrations/__init__.py")
        file_api.wipe_files(files)
        self.add_setup_imports(files, files_matching_imports)

    def init_paths_from_settings(self):
        """
        Get folder paths from settings if they exist and add forward slash if forgotten
        """
        self.core_dir = getattr(settings, "CORE_FOLDER", "")
        self.api_dir = getattr(settings, "API_FOLDER", "")
        slashed = self.core_dir.endswith("/") and self.api_dir.endswith("/")
        if len(self.api_dir) > 0 and len(self.core_dir) > 0 and not slashed:
            raise ValueError("🤔 Oops CORE_FOLDER & API_FOLDER should end with a '/'")

    def generate_app(self):
        """
        APP GENERATION
        1 - first we generate a django app through django's startapp command
        2 - we move the generated app directory to the apps directory if one is
         specified in our CLI command
        3 - we setup the files with the basic imports needed for each component
        4 - if application folder does already exist we return
        """
        core_dir_exists = path.exists(self.core_dir + self.app_name)
        api_dir_exists = path.exists(self.api_dir + self.app_name)
        if not core_dir_exists and not api_dir_exists:
            self.setup_files()

    def get_fields_string(self, fields):
        """
        get appropriate fields templates based on the field type and return them joined in a string
        """
        actual_fields = []
        relation_types = ("foreignkey", "manytomany", "onetoone")
        file = f"{self.core_dir + self.app_name}/models.py"
        for field in fields:
            field_name = field.split(":")[0]
            field_type = field.split(":")[1].lower()
            field_dict = dict(
                name=field_name,
                related=field.split(":")[2] if (field_type in relation_types) else "",
            )
            if field_type in relation_types:
                # tells developer if related model doesn't exist in file
                chunk = f'class {field_dict["related"]}(models.Model)'
                if not file_api.is_present_in_file(file, chunk):
                    print(
                        f"⚠️ bare in mind that {field_dict['related']} model doesn't exist yet!"
                    )
            field_template = model_templates.FIELD_TYPES[field_type] % field_dict
            actual_fields.append(field_template)
        return "".join(actual_fields)

    def get_model_string(self):
        """
        returns a Model class string with fields and Meta class
        """
        fields_string = self.get_fields_string(self.fields)
        params = (
            self.model_name,
            fields_string,
            pluralize(self.model_name.lower()).capitalize(),
        )
        return model_templates.MODEL % params

    def generate_models(self):
        """
        MODELS GENERATION METHODS
        1 - get_fields_string : match fields from cli with their templates and return a
         string of them joined
        2 - get_model_string : yield the fields string in the Model template and returns
         a new Model string
        3 - generate_models: check if the model does already exists in the models.py
         file if not it append it
        """
        file = f"{self.core_dir + self.app_name}/models.py"
        chunk = f"class {self.model_name}"
        if file_api.is_present_in_file(file, chunk):
            return
        model_string = self.get_model_string()
        file_api.append_file_content(file, model_string)

    def get_admin_parts(self):
        """
        returns admin Model register template and import
        """
        app_dir = self.core_dir + self.app_name
        app_path = app_dir.replace("/", ".")
        model_register_template = admin_templates.REGISTER % {"model": self.model_name}
        model_import_template = admin_templates.MODEL_IMPORT % {
            "app": app_path,
            "model": self.model_name,
        }
        return (model_import_template, model_register_template)

    def register_models_to_admin(self):
        """
        MODELS REGISTRATION TO ADMIN
        1 - get_admin_parts : returns the register template and imports of the Model
         class
        2 - register_models_to_admin : check if the model already registered in admin.py
         if not it wraps the admin file content between the Model imports and register
         template
        """
        file = f"{self.core_dir + self.app_name}/admin.py"
        chunk = f"@admin.register({self.model_name})"
        if file_api.is_present_in_file(file, chunk):
            return
        head, body = self.get_admin_parts()
        file_api.wrap_file_content(file, head, body)

    def get_viewset_parts(self):
        """
        returns viewsets templates and imports
        """
        core_app_dir = self.core_dir + self.app_name
        api_app_dir = self.api_dir + self.app_name
        core_app_path = core_app_dir.replace("/", ".")
        api_app_path = api_app_dir.replace("/", ".")
        viewset_template = view_templates.VIEWSET % {"model": self.model_name}
        model_import_template = view_templates.MODEL_IMPORT % {
            "app": core_app_path,
            "model": self.model_name,
        }
        serializer_import_template = view_templates.SERIALIZER_IMPORT % {
            "app": api_app_path,
            "model": self.model_name,
        }
        imports = model_import_template + serializer_import_template
        return (imports, viewset_template)

    def generate_views(self):
        """
        VIEWS GENERATION
        1 - get_viewset_parts : returns the viewset template and model + serializers
        imports of the Model class
        2 - generate_views : check if the viewset already exists in views.py if not
        it wraps the file content between the imports and the viewset template
        """
        file = f"{self.api_dir + self.app_name}/views.py"
        chunk = f"class {self.model_name}ViewSet"
        if file_api.is_present_in_file(file, chunk):
            return
        head, body = self.get_viewset_parts()
        file_api.wrap_file_content(file, head, body)

    def get_serializer_parts(self):
        """
        returns serializer class template and model import
        """
        app_dir = self.core_dir + self.app_name
        app_path = app_dir.replace("/", ".")
        serializer_template = serializer_templates.SERIALIZER % {
            "model": self.model_name
        }
        imports = serializer_templates.MODEL_IMPORT % {
            "app": app_path,
            "model": self.model_name,
        }
        return (imports, serializer_template)

    def generate_serializers(self):
        """
        SERIALIZER GENERATION
        1 - get_serializer_parts : returns the serializer template and imports of the Model class
        2 - generate_serializers : check if the viewset already exists in serializers.py if not
        it wraps the file content between the imports and the serializer template
        """
        serializer_file = f"{self.api_dir + self.app_name}/serializers.py"
        serializer_head = f"class {self.model_name}Serializer"
        if file_api.is_present_in_file(serializer_file, serializer_head):
            return
        head, body = self.get_serializer_parts()
        file_api.wrap_file_content(serializer_file, head, body)

    def get_url_parts(self):
        """
        returns the url template and imports of the Model class
        """
        api_app_dir = self.api_dir + self.app_name
        api_app_path = api_app_dir.replace("/", ".")
        plural_path = pluralize(self.model_name.lower())
        url_template = url_templates.URL % {
            "model": self.model_name,
            "path": plural_path,
        }
        imports = url_templates.VIEWSET_IMPORT % {
            "app": api_app_path,
            "model": self.model_name,
        }
        return (imports, url_template)

    def generate_urls(self):
        """
        URLS GENERATION
        1 - get_url_parts : returns the url template and model imports of the Model class
        2 - generate_urls : check if the url already exists in urls.py if not
        it wraps the file content between the imports and the url template, before wrapping
        it checks if the file has URL_PATTERS it take it off and append it to the url template
        """
        file = f"{self.api_dir + self.app_name}/urls.py"
        chunk = f"{self.model_name}ViewSet)"
        if file_api.is_present_in_file(file, chunk):
            return
        head, body = self.get_url_parts()
        if file_api.is_present_in_file(file, url_templates.URL_PATTERNS):
            file_api.replace_file_chunk(file, url_templates.URL_PATTERNS, "")
        body = body + url_templates.URL_PATTERNS
        file_api.wrap_file_content(file, head, body)



from setuptools import find_packages, setup

with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()

NAME = "dr_scaffold"
VERSION = "v1.4.3"
DESCRIPTION = "a Django package for scaffolding django rest apis using cli"
LONG_DESCRIPTION = (long_description,)
URL = "https://github.com/Abdenasser/dr_scaffold"
AUTHOR = "abdenasser"
AUTHOR_EMAIL = "nasser.elidrissi065@gmail.com"
LICENSE = "MIT"
PACKAGES = [
    "dr_scaffold",
    "dr_scaffold.scaffold_templates",
    "dr_scaffold.management",
    "dr_scaffold.management.commands",
]
REQUIREMENTS = ["django", "inflect"]
CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Web Environment",
    "Framework :: Django",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: POSIX :: Linux",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
]

EXCLUDE_FROM_PACKAGES = []

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    url=URL,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    install_requires=REQUIREMENTS,
    include_package_data=True,
    classifiers=CLASSIFIERS,
    zip_safe=False,
)



"""
templates for urls
"""

URL = """router.register(r'%(path)s', %(model)sViewSet)

"""

VIEWSET_IMPORT = """from %(app)s.views import %(model)sViewSet
"""

URL_PATTERNS = """urlpatterns = [
    path("", include(router.urls)),
]"""

SETUP = """from rest_framework import routers
from django.urls import include, path

router = routers.DefaultRouter()

"""



# pylint: disable=unused-argument
"""
django custom management command class
"""
from django.core.management.base import BaseCommand

from dr_scaffold.generators import Generator


class Command(BaseCommand):
    """
    django custom management command class
    """

    help = """Meant to generate Models, admin, Views, Serializers, Urls files for a ready
     to use REST api"""
    missing_args_message = (
        "You are missing some arguments in your command check the example below"
        "python manage.py dr_scaffold apps_folder/blog Article title:char body:text"
    )

    def add_arguments(self, parser):
        # need to run command as follow: python manage.py dr_scaffold app_name ModelName fields
        parser.add_argument(
            "args",
            metavar="scaffold",
            nargs="*",
            help="""Scaffold arguments (app_name
             ModelName fields).""",
        )

    def handle(self, *args, **kwargs):
        # handle the creation of an app with default files first
        generator = Generator(args[0], args[1], args[2:])
        generator.run()
